package jbmf.parser;

import jbmf.parser.constants.ConstantPool;

public class Class {

    public static final int ACC_PUBLIC = 0x0001;
    public static final int ACC_FINAL = 0x0010;
    public static final int ACC_SUPER = 0x0020;
    public static final int ACC_INTERFACE = 0x0200;
    public static final int ACC_ABSTRACT = 0x0400;
    public static final int ACC_SYNTHETIC = 0x1000;
    public static final int ACC_ANNOTATION = 0x2000;
    public static final int ACC_ENUM = 0x4000;

    public static final Class INVALID_CLASS = new Class();

    private int minorVersion;
    private int majorVersion;
    private ConstantPool constPool;
    private int access;


    public int getMajorVersion() {
        return majorVersion;
    }

    public void setMajorVersion(int majorVersion) {
        this.majorVersion = majorVersion;
    }

    public int getMinorVersion() {
        return minorVersion;
    }

    public void setMinorVersion(int minorVersion) {
        this.minorVersion = minorVersion;
    }

    public ConstantPool getConstPool() {
        return constPool;
    }

    public void setConstPool(ConstantPool constPool) {
        this.constPool = constPool;
    }

    public int getAccess() {
        return access;
    }

    public void setAccess(int access) {
        this.access = access;
    }

    public boolean isPublic() {
        return (access & ACC_PUBLIC) == ACC_PUBLIC;
    }

    public boolean isFinal() {
        return (access & ACC_FINAL) == ACC_FINAL;
    }

    public boolean isSuper() {
        return (access & ACC_SUPER) == ACC_SUPER;
    }

    public boolean isInterface() {
        return (access & ACC_INTERFACE) == ACC_INTERFACE;
    }

    public boolean isClass() {
        return !isInterface();
    }

    public boolean isAbstract() {
        return (access & ACC_ABSTRACT) == ACC_ABSTRACT;
    }

    public boolean isSynthetic() {
        return (access & ACC_SYNTHETIC) == ACC_SYNTHETIC;
    }

    public boolean isAnnotation() {
        return (access & ACC_ANNOTATION) == ACC_ANNOTATION;
    }

    public boolean isEnum() {
        return (access & ACC_ENUM) == ACC_ENUM;
    }
}
