package jbmf.parser;


import jbmf.parser.constants.ConstantPool;
import jbmf.utils.ByteUtil;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

public class ClassReader {

    public static final int MAGIC = 0xCAFEBABE;

    private ClassReader() {}

    public static Class get(InputStream is) {
        Class clazz = new Class();
        byte[] buffer = new byte[100];

        try {
            is.read(buffer, 0, 10);

            if(ByteUtil.readU4(buffer, 0) != MAGIC) {
                return Class.INVALID_CLASS;
            }

            clazz.setMinorVersion(ByteUtil.readU2(buffer, 4));
            debug("Minor version = " + clazz.getMinorVersion());
            clazz.setMajorVersion(ByteUtil.readU2(buffer, 6));
            debug("Major version = " + clazz.getMajorVersion());

            clazz.setConstPool(new ConstantPool(is, buffer));

            is.read(buffer, 0, 8);
            clazz.setAccess(ByteUtil.readU2(buffer, 0));
            debug("Access modifiers: " + clazz.getAccess());
            validateAccess(clazz);



            is.close();
        } catch (IOException e1) {
            return Class.INVALID_CLASS;
        }

        return clazz;
    }

    public static Class get(String path) {
        try {
            return get(new FileInputStream(path));
        } catch (FileNotFoundException e) {
            return Class.INVALID_CLASS;
        }
    }

    //todo this is somewhat inefficient
    public static boolean validateAccess(Class clazz) {
        if(clazz.isInterface()) {
            assert clazz.isAbstract();
            assert !clazz.isEnum();
            assert !clazz.isSuper();
            assert !clazz.isFinal();
        }

        if(clazz.isAnnotation()) {
            assert clazz.isInterface();
        }

        if(clazz.isClass()) {
            assert !clazz.isAnnotation();
            assert !(clazz.isFinal() && clazz.isAbstract());
        }

        return true;
    }

    private static void debug(String msg) {
        System.out.println(msg);
    }

}
