package jbmf.parser.constants;

import jbmf.utils.ByteUtil;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;

public class ConstantPool implements Constants {

    private byte[] buffer;
    private InputStream is;
    private int size;
    private Constant[] constants;

    public ConstantPool(InputStream is, byte[] buffer) {
        this.is = is;
        this.buffer = buffer;
        try {
            parse();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public ConstantPool(Constant[] constants) {
        this.constants = constants;
        this.size = constants.length;
    }

    public ConstantPool(InputStream is) {
        this(is, new byte[100]);
    }

    public Constant[] getConstants() {
        return constants;
    }

    private void parse() throws IOException {
        size = ByteUtil.readU2(buffer, 8);
        constants = new Constant[size];
        System.out.println("Const pool entries: " + size);

        for(int i = 1; i < size; i++) {
            is.read(buffer, 0, 1);
            int tag = ByteUtil.readU1(buffer, 0);
            byte[] data;

            switch(tag) {
                case CONSTANT_CLASS:
                    data = new byte[CONSTANT_CLASS_SIZE];
                    is.read(data, 0, CONSTANT_CLASS_SIZE);
                    constants[i] = new ClassConstant(data);
                    break;

                case CONSTANT_DOUBLE:
                    data = new byte[CONSTANT_DOUBLE_SIZE];
                    is.read(data, 0, CONSTANT_DOUBLE_SIZE);
                    constants[i] = new DoubleConstant(data);
                    i++;
                    break;

                case CONSTANT_FIELDREF:
                    data = new byte[CONSTANT_FIELDREF_SIZE];
                    is.read(data, 0, CONSTANT_FIELDREF_SIZE);
                    constants[i] = new FieldRefConstant(data);
                    break;

                case CONSTANT_FLOAT:
                    data = new byte[CONSTANT_FLOAT_SIZE];
                    is.read(data, 0, CONSTANT_FLOAT_SIZE);
                    constants[i] = new FloatConstant(data);
                    break;

                case CONSTANT_IFACEMTHODREF:
                    data = new byte[CONSTANT_IFACEMETHODREF_SIZE];
                    is.read(data, 0, CONSTANT_IFACEMETHODREF_SIZE);
                    constants[i] = new IFaceMethodRefConstant(data);
                    break;

                case CONSTANT_INTEGER:
                    data = new byte[CONSTANT_INTEGER_SIZE];
                    is.read(data, 0, CONSTANT_INTEGER_SIZE);
                    constants[i] = new IntegerConstant(data);
                    break;

                case CONSTANT_INVOKEDYNAMIC:
                    data = new byte[CONSTANT_INVOKEDYNAMIC_SIZE];
                    is.read(data, 0, CONSTANT_INVOKEDYNAMIC_SIZE);
                    constants[i] = new InvokeDynamicConstant(data);
                    break;

                case CONSTANT_LONG:
                    data = new byte[CONSTANT_LONG_SIZE];
                    is.read(data, 0, CONSTANT_LONG_SIZE);
                    constants[i] = new LongConstant(data);
                    i++;
                    break;

                case CONSTANT_METHODHANDLE:
                    data = new byte[CONSTANT_METHODHANDLE_SIZE];
                    is.read(data, 0, CONSTANT_METHODHANDLE_SIZE);
                    constants[i] = new MethodHandleConstant(data);
                    break;

                case CONSTANT_METHODREF:
                    data = new byte[CONSTANT_METHODREF_SIZE];
                    is.read(data, 0, CONSTANT_METHODREF_SIZE);
                    constants[i] = new MethodRefConstant(data);
                    break;

                case CONSTANT_METHODTYPE:
                    data = new byte[CONSTANT_METHODTYPE_SIZE];
                    is.read(data, 0, CONSTANT_METHODTYPE_SIZE);
                    constants[i] = new MethodTypeConstant(data);
                    break;

                case CONSTANT_NAMEANDTYPE:
                    data = new byte[CONSTANT_NAMEANDTYPE_SIZE];
                    is.read(data, 0, CONSTANT_NAMEANDTYPE_SIZE);
                    constants[i] = new NameAndTypeConstant(data);
                    break;

                case CONSTANT_STRING:
                    data = new byte[CONSTANT_STRING_SIZE];
                    is.read(data, 0, CONSTANT_STRING_SIZE);
                    constants[i] = new StringConstant(data);
                    break;

                case CONSTANT_UTF8:
                    is.read(buffer, 0, 2);
                    int len = ByteUtil.readU2(buffer, 0);
                    data = new byte[len];
                    is.read(data, 0, len);
                    constants[i] = new UTF8Constant(data);
                    break;

                default:
                    System.out.println("Invalid constants pool tag: " + tag);
                    break;
            }
        }
    }

    public void dump(PrintStream out) {
        StringBuilder s = new StringBuilder();

        int i = 1;
        for(Constant cons : constants) {
            if(cons != null) {
                s.append("["+i+"] " + cons + System.lineSeparator());
            }
            i++;
        }

        out.print(s);
    }

    public void dump() {
        dump(System.out);
    }
}
