package jbmf.utils;


public class ByteUtil {

    public static long readU8(byte[] bytes, int offset) {
        assert bytes != null;
        assert bytes.length >= offset + 8;
        assert offset > -1;

        return ((long)(bytes[offset] & 0xFF) << 56) | ((long)(bytes[offset + 1] & 0xFF) << 48)
                | ((long)(bytes[offset + 2] & 0xFF) << 40) | ((long)(bytes[offset + 3] & 0xFF) << 32)
                | ((long)(bytes[offset + 4] & 0xFF) << 24) | ((long)(bytes[offset + 5] & 0xFF) << 16)
                | ((long)(bytes[offset + 6] & 0xFF) << 8) | ((long)bytes[offset + 7] & 0xFF);
    }

    public static int readU4(byte[] bytes, int offset) {
        assert bytes != null;
        assert bytes.length >= offset + 4;
        assert offset > -1;

        return ((bytes[offset] & 0xFF) << 24) | ((bytes[offset + 1] & 0xFF) << 16)
                | ((bytes[offset + 2] & 0xFF) << 8) | (bytes[offset + 3] & 0xFF);
    }

    public static int readU2(byte[] bytes, int offset) {
        assert bytes != null;
        assert bytes.length >= offset + 2;
        assert offset > -1;

        return ((bytes[offset] & 0xFF) << 8) | (bytes[offset + 1] & 0xFF);
    }

    public static int readU1(byte[] bytes, int offset) {
        assert bytes != null;
        assert bytes.length >= offset + 1;
        assert offset > -1;

        return bytes[offset] & 0xFF;
    }

    public static void writeU4(byte[] bytes, int offset, int value) {
        assert bytes != null;
        assert bytes.length >= offset + 4;
        assert offset > -1;

        bytes[offset] = (byte) ((value >> 24) & 0xFF);
        bytes[offset + 1] = (byte) ((value >> 16) & 0xFF);
        bytes[offset + 2] = (byte) ((value >> 8) & 0xFF);
        bytes[offset + 3] = (byte) (value & 0xFF);
    }

    public static void writeU2(byte[] bytes, int offset, int value) {
        assert bytes != null;
        assert bytes.length >= offset + 2;
        assert offset > -1;

        bytes[offset] = (byte) ((value >> 8) & 0xFF);
        bytes[offset + 1] = (byte) (value & 0xFF);
    }

    public static void writeU8(byte[] bytes, int offset, long value) {
        assert bytes != null;
        assert bytes.length >= offset + 8;
        assert offset > -1;

        bytes[offset] = (byte) ((value >> 56) & 0xFF);
        bytes[offset + 1] = (byte) ((value >> 48) & 0xFF);
        bytes[offset + 2] = (byte) ((value >> 40) & 0xFF);
        bytes[offset + 3] = (byte) ((value >> 32) & 0xFF);
        bytes[offset + 4] = (byte) ((value >> 24) & 0xFF);
        bytes[offset + 5] = (byte) ((value >> 16) & 0xFF);
        bytes[offset + 6] = (byte) ((value >> 8) & 0xFF);
        bytes[offset + 7] = (byte) (value & 0xFF);
    }

    public static void writeU1(byte[] bytes, int offset, int referenceKind) {
        assert bytes != null;
        assert bytes.length >= offset + 1;
        assert offset > -1;

        bytes[offset] = (byte) (referenceKind & 0xFF);
    }
}
